package com.reactlibrary.activity;

import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.baidu.location.BDLocation;
import com.baidu.mapapi.map.BaiduMap;
import com.baidu.mapapi.map.BitmapDescriptor;
import com.baidu.mapapi.map.BitmapDescriptorFactory;
import com.baidu.mapapi.map.DotOptions;
import com.baidu.mapapi.map.InfoWindow;
import com.baidu.mapapi.map.MapPoi;
import com.baidu.mapapi.map.MapStatus;
import com.baidu.mapapi.map.MapStatusUpdate;
import com.baidu.mapapi.map.MapStatusUpdateFactory;
import com.baidu.mapapi.map.MapView;
import com.baidu.mapapi.map.MarkerOptions;
import com.baidu.mapapi.map.MyLocationConfiguration;
import com.baidu.mapapi.map.MyLocationData;
import com.baidu.mapapi.map.OverlayOptions;
import com.baidu.mapapi.map.Polyline;
import com.baidu.mapapi.map.PolylineOptions;
import com.baidu.mapapi.map.TextOptions;
import com.baidu.mapapi.map.UiSettings;
import com.baidu.mapapi.model.LatLng;
import com.baidu.mapapi.model.LatLngBounds;
import com.baidu.mapapi.navi.BaiduMapAppNotSupportNaviException;
import com.baidu.mapapi.navi.BaiduMapNavigation;
import com.baidu.mapapi.navi.NaviParaOption;
import com.facebook.react.ReactActivity;
import com.facebook.react.ReactInstanceManager;
import com.facebook.react.ReactPackage;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableMap;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.reactlibrary.R;
import com.reactlibrary.RNReactNativeBaiduMapLocationModule;
import com.reactlibrary.RNReactNativeBaiduMapLocationPackage;
import com.reactlibrary.listener.UpdateLocationListener;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import static com.reactlibrary.activity.MapParamEx.ERROR_VALUE;

public class BaiduMapActivity extends ReactActivity implements UpdateLocationListener, View.OnClickListener {
    private static final String TAG = "BaiduMapActivity";
    private static final String TYPE_SELECT_LOCATION = "typeSelectLocation";
    /*线*/
    private final static String LINESTRING = "LineString";
    /*面*/
    private final static String POLYGON = "Polygon";

    private BDLocation currentBDLocation;
    private MapView mMapView;
    private ImageView ivSelectPoint, ivMapMark;
    private ImageButton ivMapNavigation, ivCurrentLocation;
    private BaiduMap mBaiduMap;
    private int mMapStatus;
    private ReadableMap openActivityOptionMap;
    private LatLng mDestLatLng;
    private RNReactNativeBaiduMapLocationModule baiduMapLocationModule;
    private LatLngBounds mMapBounds;
    private boolean mInitSuccess = false;

    private final static double MAP_MIN_LONGITUDE = 73.5;
    private final static double MAP_MAX_LONGITUDE = 135.0;
    private final static double MAP_MIN_LATITUDE = 4.0;
    private final static double MAP_MAX_LATITUDE = 53.0;
    private double defaultLongitude = 121.525563;
    private double defaultLatitude = 31.28061;
    private float mZoomLevel;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_baidu_map);
        findView();
        getBaiduMapLocationModule();
        initMap();
        setViewListener();
    }

    @Override
    protected void onResume() {
        super.onResume();
        //在activity执行onResume时执行mMapView. onResume ()，实现地图生命周期管理
        mMapView.onResume();
        if (baiduMapLocationModule != null) baiduMapLocationModule.registerUpdateLocationListener(this);
    }

    @Override
    protected void onPause() {
        super.onPause();
        //在activity执行onPause时执行mMapView. onPause ()，实现地图生命周期管理
        if (baiduMapLocationModule != null) baiduMapLocationModule.unregisterUpdateLocationListener();
        mMapView.onPause();
    }
    @Override
    protected void onDestroy() {
        super.onDestroy();
        //在activity执行onDestroy时执行mMapView.onDestroy()，实现地图生命周期管理
        mMapView.onDestroy();
        mBaiduMap.setMyLocationEnabled(false);
    }

    @Override
    public void onClick(View v) {
        int id = v.getId();
        if (id == R.id.iv_select_point) {
            selectPoint();
        } else if (id == R.id.iv_map_mark) {
        } else if (id == R.id.iv_map_navigation) {
            navigate();
        } else if (id == R.id.iv_current_location) {
            location();
        }
    }

    @Override
    public void updateLocation(BDLocation location) {
        Log.i(TAG, location.toString());
        currentBDLocation = location;
        MyLocationData locData = new MyLocationData.Builder()
                .accuracy(location.getRadius())
                // 此处设置开发者获取到的方向信息，顺时针0-360
                .direction(location.getDirection()).latitude(location.getLatitude())
                .longitude(location.getLongitude()).build();
        mBaiduMap.setMyLocationData(locData);
    }

    private void selectPoint() {
        if (baiduMapLocationModule == null) return;
        if (!mInitSuccess) {
            Toast.makeText(this, R.string.text_select_coordinate_failure, Toast.LENGTH_SHORT).show();
            return;
        }

        MapStatus mapStatus = mBaiduMap.getMapStatus();
        LatLng latLng = mapStatus.target;
        WritableMap map = Arguments.createMap();
        map.putDouble(MapParamEx.LONGITUDE, latLng.longitude);
        map.putDouble(MapParamEx.LATITUDE, latLng.latitude);
        baiduMapLocationModule.sendEvent(TYPE_SELECT_LOCATION, map);
        finish();
    }

    private void navigate() {
        if (!mInitSuccess || currentBDLocation == null || mDestLatLng == null) {
            Toast.makeText(this, R.string.text_navigate_failure, Toast.LENGTH_LONG).show();
            return;
        }

        // 构建 导航参数
        NaviParaOption naviParaOption = new NaviParaOption();
        LatLng startLatLng = new LatLng(currentBDLocation.getLatitude(), currentBDLocation.getLongitude());
        naviParaOption.startPoint(startLatLng);
        naviParaOption.startName(getString(R.string.text_start_point));
        naviParaOption.endPoint(mDestLatLng);
        naviParaOption.endName(getResources().getString(R.string.text_end_point));
        try {
            BaiduMapNavigation.openBaiduMapNavi(naviParaOption, this);
        } catch (BaiduMapAppNotSupportNaviException e) {
            e.printStackTrace();
            Toast.makeText(this, R.string.text_navigate_failure_message, Toast.LENGTH_LONG).show();
        }
    }

    private void location() {
        if (currentBDLocation == null) return;
        LatLng GEO_SHANGHAI = new LatLng(currentBDLocation.getLatitude(), currentBDLocation.getLongitude());
        MapStatusUpdate status = MapStatusUpdateFactory.newLatLng(GEO_SHANGHAI);
        mBaiduMap.animateMapStatus(status);
        ivMapMark.setVisibility(View.INVISIBLE);
    }

    private void findView() {
        mMapView = findViewById(R.id.map_view);
        ivSelectPoint = findViewById(R.id.iv_select_point);
        ivMapMark = findViewById(R.id.iv_map_mark);
        ivMapNavigation = findViewById(R.id.iv_map_navigation);
        ivCurrentLocation = findViewById(R.id.iv_current_location);
    }

    private void initMap() {
        mBaiduMap = mMapView.getMap();
        //普通地图
        mBaiduMap.setMapType(BaiduMap.MAP_TYPE_NORMAL);
        mZoomLevel = mBaiduMap.getMaxZoomLevel();

        UiSettings uiSettings = mBaiduMap.getUiSettings();
        uiSettings.setOverlookingGesturesEnabled(false);
        uiSettings.setRotateGesturesEnabled(false);

        initMapStatus();

        mBaiduMap.setMyLocationEnabled(true);
        // 设置定位图层的配置
        MyLocationConfiguration config = new MyLocationConfiguration(MyLocationConfiguration.LocationMode.NORMAL,
                true, null);
        mBaiduMap.setMyLocationConfigeration(config);

        // add map listeners
        mBaiduMap.setOnMapStatusChangeListener(mOnMapStatusChangeListener);
        mBaiduMap.setOnMapClickListener(mOnMapClickListener);
        //mBaiduMap.setOnMarkerClickListener(mOnMarkerClickListener);
        mBaiduMap.setOnMapLoadedCallback(mOnMapLoadedCallback);
    }

    private void initMapStatus() {
        if (baiduMapLocationModule != null) openActivityOptionMap = baiduMapLocationModule.getOpenActivityOptionMap();
        if (openActivityOptionMap == null || !openActivityOptionMap.hasKey(MapParamEx.MAP_STATUS)) {
            MapStatusUpdate u = MapStatusUpdateFactory.zoomTo(mZoomLevel);
            mBaiduMap.setMapStatus(u);
            return;
        }

        mMapStatus = openActivityOptionMap.getInt(MapParamEx.MAP_STATUS);

        double longitude = openActivityOptionMap.hasKey(MapParamEx.LONGITUDE)
                ? openActivityOptionMap.getDouble(MapParamEx.LONGITUDE) : defaultLongitude;
        double latitude = openActivityOptionMap.hasKey(MapParamEx.LATITUDE)
                ? openActivityOptionMap.getDouble(MapParamEx.LATITUDE) : defaultLatitude;
        initMapStatusNext(latitude, longitude);
    }

    private void initMapStatusNext(double latitude, double longitude) {
        LatLng destLatLng = new LatLng(latitude, longitude);

        LatLng pt1 = new LatLng(MAP_MIN_LATITUDE, MAP_MIN_LONGITUDE);
        LatLng pt2 = new LatLng(MAP_MAX_LATITUDE, MAP_MAX_LONGITUDE);

        mMapBounds = new LatLngBounds.Builder().include(pt1).include(pt2).build();
        if (!mMapBounds.contains(destLatLng)) {
            if (currentBDLocation != null) {
                destLatLng = new LatLng(currentBDLocation.getLatitude(), currentBDLocation.getLongitude());
            } else {
                longitude = defaultLongitude;
                latitude = defaultLatitude;
                destLatLng = new LatLng(latitude, longitude);
            }

            if (!mMapBounds.contains(destLatLng)) {
                Toast.makeText(this, R.string.text_coordinate_invalid, Toast.LENGTH_SHORT).show();
                return;
            }
        }

        MapStatusUpdate u = MapStatusUpdateFactory.newLatLngZoom(destLatLng, mZoomLevel);
        mBaiduMap.setMapStatus(u);
        switch (mMapStatus) {
            case MapStatusEx.VIEW_MAP:
                break;
            case MapStatusEx.LOCATE_MAP:
                if (openActivityOptionMap.hasKey(MapParamEx.ADDRESS))
                    initInfoWindow(destLatLng, openActivityOptionMap.getString(MapParamEx.ADDRESS));
                break;
            case MapStatusEx.MARK_MAP:
                ivMapMark.setVisibility(View.VISIBLE);
                ivSelectPoint.setVisibility(View.VISIBLE);
                break;
            case MapStatusEx.TRACK_MAP:
                if (openActivityOptionMap.hasKey(MapParamEx.TRACE_INFO_LIST))
                    initTrackMap(openActivityOptionMap.getArray(MapParamEx.TRACE_INFO_LIST));
                break;
            case MapStatusEx.VIEW_ADDRESS:
                initAddressView();
                break;
            default:
                break;
        }
    }

    // 地图状态改变相关接口
    private BaiduMap.OnMapStatusChangeListener mOnMapStatusChangeListener =
            new BaiduMap.OnMapStatusChangeListener() {
                @Override
                public void onMapStatusChangeStart(com.baidu.mapapi.map.MapStatus mapStatus) {

                }

                @Override
                public void onMapStatusChangeStart(MapStatus var1, int var2) {

                }

                @Override
                public void onMapStatusChange(com.baidu.mapapi.map.MapStatus mapStatus) {

                }

                @Override
                public void onMapStatusChangeFinish(com.baidu.mapapi.map.MapStatus mapStatus) {
                    mZoomLevel = mapStatus.zoom;  //地图缩放等级
                }
            };

    // 地图单击事件监听接口
    private BaiduMap.OnMapClickListener mOnMapClickListener =
            new BaiduMap.OnMapClickListener() {
                @Override
                public void onMapClick(LatLng latLng) { }

                @Override
                public boolean onMapPoiClick(MapPoi mapPoi) {
                    return false;
                }
            };

    // 地图加载完成回调接口
    private BaiduMap.OnMapLoadedCallback mOnMapLoadedCallback = new BaiduMap.OnMapLoadedCallback(){
        @Override
        public void onMapLoaded() {
            mInitSuccess = true;
        }
    };

    private void setViewListener() {
        ivSelectPoint.setOnClickListener(this);
        ivMapMark.setOnClickListener(this);
        ivMapNavigation.setOnClickListener(this);
        ivCurrentLocation.setOnClickListener(this);
    }

    private void initInfoWindow(LatLng latLng, String address) {
        if ((latLng == null) || address == null) return;
        View v = getLayoutInflater().inflate(R.layout.custom_text_view, null);
        TextView popupText = v.findViewById(R.id.textcache);
        popupText.setText(address);
        InfoWindow mInfoWindow = new InfoWindow(v, latLng, 0);
        mBaiduMap.showInfoWindow(mInfoWindow);
        mDestLatLng = latLng;
        ivMapNavigation.setVisibility(View.VISIBLE);
    }

    private void initTrackMap(ReadableArray array) {
        if (array == null || array.size() == 0) return;
        int size = array.size();
        ReadableMap map;
        int color1, color2, color3;
        int zIndex = 1;
        double longitude, latitude;
        LatLng latLng;
        LatLngBounds.Builder builder = new LatLngBounds.Builder();
        List<LatLng> oneList = new ArrayList<>();
        List<LatLng> twoList = new ArrayList<>();
        for (int i = 0; i < size; i++) {
            map = array.getMap(i);
            if (map == null) continue;
            longitude = map.getDouble(MapParamEx.LONGITUDE);
            latitude = map.getDouble(MapParamEx.LATITUDE);
            if (Math.abs(longitude) <= ERROR_VALUE || Math.abs(latitude) <= ERROR_VALUE) continue;
            latLng = new LatLng(latitude, longitude);
            if (!mMapBounds.contains(latLng)) continue;
            if (map.getBoolean(MapParamEx.RECORDED)) { // 已抄
                color1 = 0xFFFF0000;
                color2 = 0x00FFFFFF;
                color3 = 0xFFFFFFFF;
                oneList.add(latLng);
            } else { // 未抄
                color1 = 0xFFFFFF00;
                color2 = 0x00FFFFFF;
                color3 = 0xFF000000;
                twoList.add(latLng);
            }
            OverlayOptions dotOptions = new DotOptions()
                    .center(latLng)
                    .radius(20)
                    .color(color1)
                    .zIndex(zIndex);
            mBaiduMap.addOverlay(dotOptions);

            OverlayOptions textOption = new TextOptions()
                    .bgColor(color2)
                    .fontSize(20)
                    .fontColor(color3)
                    .text(Integer.toString(map.getInt(MapParamEx.ORDER_NUMBER)))
                    .position(latLng)
                    .zIndex(zIndex + 1);
            mBaiduMap.addOverlay(textOption);
            builder = builder.include(latLng);
        }

        if (oneList.size() > 1) {
            PolylineOptions onePolylineOptions = new PolylineOptions()
                    .color(0xFFFF0000)
                    .points(oneList);
            mBaiduMap.addOverlay(onePolylineOptions);
        }

        if (twoList.size() > 1) {
            PolylineOptions twoPolylineOptions = new PolylineOptions()
                    .color(0xFFFFFF00)
                    .points(twoList);
            mBaiduMap.addOverlay(twoPolylineOptions);
        }

        LatLngBounds bounds = builder.build();
        MapStatusUpdate u = MapStatusUpdateFactory.newLatLngBounds(bounds);
        mBaiduMap.animateMapStatus(u);
    }

    private void initAddressView() {
        Intent intent = getIntent();
        String pointAddress = intent.getStringExtra("keyPointAddress");
        try {
            ArrayList<KeyPoint> keyPointList = new Gson().fromJson(pointAddress, new TypeToken<ArrayList<KeyPoint>>() {}.getType());
            if (keyPointList != null && keyPointList.size() > 0) {
                List<OverlayOptions> options = new ArrayList<>();
                for (int i = 0; i < keyPointList.size(); i++) {
                    KeyPoint keyPoint = keyPointList.get(i);
                    LatLng latLng = new LatLng(keyPoint.getLatitude(), keyPoint.getLongitude());
                    BitmapDescriptor bitmap;
                    if (keyPoint.getPointStatus() == 1) {//已打卡
                        bitmap = BitmapDescriptorFactory
                                .fromResource(R.drawable.ic_mark);
                    } else {//已经打卡
                        bitmap = BitmapDescriptorFactory
                                .fromResource(R.drawable.ic_un_mark);
                    }
                    OverlayOptions option = new MarkerOptions()
                            .position(latLng)
                            .icon(bitmap);
                    options.add(option);
                }
                mBaiduMap.addOverlays(options); //添加标签
            }
        } catch (Exception e) {
            Toast.makeText(this, e.getMessage(), Toast.LENGTH_SHORT).show();
        }

        // 绘制工单地址的点线面
        String orderAddress = intent.getStringExtra("orderAddress");
        try {
            OrderAddress addresses = new Gson().fromJson(orderAddress, new TypeToken<OrderAddress>() {
            }.getType());
            if (addresses != null) {
                switch (addresses.getType()) {
                    case BaiduMapActivity.POLYGON:
                    case BaiduMapActivity.LINESTRING: //线
                        List<LatLng> points = new ArrayList<>();
                        ArrayList<KeyPoint> keyPoints = addresses.getPoints();
                        for (int i = 0; i < keyPoints.size(); i++) {
                            KeyPoint point = keyPoints.get(i);
                            LatLng p1 = new LatLng(point.getLatitude(), point.getLongitude());
                            points.add(p1);
                        }
                        OverlayOptions ooPolyline = new PolylineOptions().width(10)
                                .color(0xAA00FF00).points(points);
                        Polyline mPolyline = (Polyline) mBaiduMap.addOverlay(ooPolyline);
                        mPolyline.setDottedLine(true);
                        break;
                    default:  //点
                        LatLng latLng = new LatLng(addresses.getPoints().get(0).getLatitude(),
                                addresses.getPoints().get(0).getLongitude());
                        BitmapDescriptor bitmap = BitmapDescriptorFactory.fromResource(R.drawable.ic_dest);
                        OverlayOptions option = new MarkerOptions()
                                .position(latLng)
                                .icon(bitmap);
                        //在地图上添加Marker，并显示
                        mBaiduMap.addOverlay(option);
                        break;
                }
            }
        } catch (Exception e) {
            Toast.makeText(this, e.getMessage(), Toast.LENGTH_SHORT).show();
        }
    }

    private void getBaiduMapLocationModule() {
        try {
            ReactInstanceManager manager = getReactInstanceManager();
            Class managerClass = manager.getClass();
            Field field = managerClass.getDeclaredField("mPackages");
            field.setAccessible(true);
            List<ReactPackage> list = (List<ReactPackage>) field.get(manager);
            ReactPackage baiduMapLocationPackage = null;
            for (ReactPackage viewManager : list) {
                if (viewManager instanceof RNReactNativeBaiduMapLocationPackage) {
                    baiduMapLocationPackage = viewManager;
                    break;
                }
            }
            if (baiduMapLocationPackage == null) return;
            Class baiduMapLocationPackageClass = baiduMapLocationPackage.getClass();
            Field baiduMapLocationPackageField = baiduMapLocationPackageClass.getDeclaredField("baiduMapLocationModule");
            baiduMapLocationPackageField.setAccessible(true);
            baiduMapLocationModule = (RNReactNativeBaiduMapLocationModule) baiduMapLocationPackageField.get(baiduMapLocationPackage);
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
    }

}
